# 3.7 Numerical Double Integration Test.R
# function(), plot(), lines(), legend(), integrate()
# Normal and lognormal statistics
# rmarkdown::render("3.7 Numerical Integration Test.R", "word_document")
rm(list = ls())  # Take out the Environment "trash"
cat("\014") # Clear Console, making error checking easier.
while (!is.null(dev.list()))  dev.off() # Clear old plots
par(family = 'Times New Roman') # Globally set fonts for graphs
# Libraries
#  stats: integrate(), single integral
#  pbivnorm: pbivnorm(), standard bivariate normal distribution
#  mvtnorm: pmvnorm(), multivariate normal CDF for arbitrary limits and rho
#  pracma: integral2(), double integral
#  plot3D: mesh()
Packages <- c("stats", "pbivnorm", "mvtnorm", "pracma") 
if(length(setdiff(Packages, rownames(installed.packages()))) > 0) {
  install.packages(setdiff(Packages, rownames(installed.packages())))
} # Make sure libraries are installed on this computer
lapply(Packages, library, character.only=TRUE) # Load and attach libraries
rm(Packages)
# Test Data
StockPrice = 100.0
StrikePrice = 100.0
InterestRate = 12.0
DividendYield = 0.0
Volatility = 30.0
TimeToMaturity = 1.0
source('Normal Distribution Functions.R')
source('Lognormal Distribution Functions.R')
#
# Double integral tests
#
a = 0
b = 0
rho1 = 0
# Standard bivariate normal CDF, pbivnorm(), bivariate only
N2CDFTest1 <- pbivnorm(x = a, y = b, rho = rho1, recycle = TRUE)
N2CDFTest1
# Standard bivariate normal CDF, pmvnorm(), multivariate function
mean1 <- rep(0,2)
lower1 <- rep(-Inf,2)
corr1 <- diag(2)
corr1[lower.tri(corr1)] <- 0.0
corr1[upper.tri(corr1)] <- 0.0
upper1 <- c(0,0)
N2CDFTest2 <- pmvnorm(lower=lower1, upper=upper1, mean=mean1, corr=corr1)[1]
N2CDFTest2
# Single integral, univariate normal CDF
N1CDFTest1 <- integrate(NormalPDF, -Inf, 0, 0, 1)[1]
N1CDFTest1
#
# Double integral, two univariate normal CDF
#
Normal2PDF <- function(x1, x2, Mu1, Mu2, SD1, SD2, rho){
  z1 <- (x1 - Mu1) / SD1
  z2 <- (x2 - Mu2) / SD2
  z <- z1^2 -2*rho*z1*z2 + z2^2
  PDF <- ( 1.0 / (2.0*pi*SD1*SD2*sqrt(1-rho^2)) ) * 
    exp( -(z / (2.0*sqrt(1 - rho^2))) )
  return(PDF)
}
# Example
Mu1 <- 0
Mu2 <- 0
SD1 <- 1
SD2 <- 1
rho <- 0
x1min <- Mu1 - 5*SD1
x2min <- Mu2 - 5*SD2
N2CDFTest1 <- integral2(Normal2PDF, xmin = x1min, xmax = 0, ymin = x2min, 
  ymax = 0, reltol = 1e-6, Mu1 = Mu1, Mu2 = Mu2, SD1 = SD1, SD2 = SD2, 
  rho = rho)[1]
N2CDFTest1

NX1Obs <- 51
NX2Obs <- 51
Mu1 <- 0
Mu2 <- 0
SD1 <- 1
SD2 <- 1
rho <- 0
MinX1 <- Mu1 - 5*SD1
MinX2 <- Mu2 - 5*SD2
MaxX1 <- Mu1 + 5*SD1
MaxX2 <- Mu2 + 5*SD2
X1Incr <- (MaxX1 - MinX1) / (NX1Obs - 1)
X2Incr <- (MaxX2 - MinX2) / (NX2Obs - 1)
N2CDFV1 <- array(data = -99, dim = c(NX1Obs, NX2Obs)) # integral2()
mean1 <- rep(0,2)
mean1[1] <- Mu1
mean1[2] <- Mu2
lower1 <- rep(-Inf,2)
lower1[1] <- MinX1
lower1[2] <- MinX2
corr1 <- diag(2)
corr1[lower.tri(corr1)] <- rho
corr1[upper.tri(corr1)] <- rho
upper1 <- c(0,0)
N2CDFV2 <- array(data = -99, dim = c(NX1Obs, NX2Obs)) # pmvnorm()
N2CDFV3 <- array(data = -99, dim = c(NX1Obs, NX2Obs)) # pbivnorm() 
N2PDFV1 <- array(data = -99, dim = c(NX1Obs, NX2Obs))
# Machine error analysis
x1 <- as.numeric(array(data = NA, dim = c(NX1Obs)))
x2 <- as.numeric(array(data = NA, dim = c(NX2Obs)))
Error12 <- array(data = 0, dim = c(NX1Obs, NX2Obs)) # integral - pmvnorm
Error13 <- array(data = 0, dim = c(NX1Obs, NX2Obs)) # integral - pbivnorm
Error23 <- array(data = 0, dim = c(NX1Obs, NX2Obs)) # pmvnorm - pbivnorm
for(i in 1:NX1Obs){
  for(j in 1:NX2Obs){
    x1[i] <- MinX1 + X1Incr * i
    x2[j] <- MinX2 + X2Incr * j
    MaxX1 <- as.numeric(x1[i])
    MaxX2 <- as.numeric(x2[j])
    N2CDFV1[i, j] <- as.numeric(integral2(Normal2PDF, xmin = MinX1, xmax = MaxX1,
      ymin=MinX2, ymax = MaxX2, reltol = 1e-6, Mu1 = Mu1, Mu2 = Mu2,
      SD1 = SD1, SD2 = SD2, rho = rho)[1])
    N2PDFV1[i,j] <- Normal2PDF(x1[i], x2[j], Mu1, Mu2, SD1, SD2, rho)
    upper1[1] <- MaxX1
    upper1[2] <- MaxX2
    N2CDFV2[i, j] <- pmvnorm(lower=lower1, upper=upper1, mean=mean1, corr=corr1)[1]
# pbivnorm(): Only STANDARD bivariate normal CDF
    N2CDFV3[i, j] <- pbivnorm(x = MaxX1, y = MaxX2, rho = rho, recycle = TRUE)
    Error12[i, j] <-  N2CDFV1[i, j] - N2CDFV2[i, j]
    Error13[i, j] <-  N2CDFV1[i, j] - N2CDFV3[i, j]
    Error23[i, j] <-  N2CDFV2[i, j] - N2CDFV3[i, j]
  }
}
# 3D graphs of machine error
M <- meshgrid(x1, x2)
x3 <- x1
y3 <- x2
mTitle1 <- "Bivariate CDF (integral2)" 
phi1 <- 45
theta1 <- 45
MaxXValue = max(x3)
MinXValue = min(x3)
xlim1 = c(1:2); xlim1[1] = MinXValue; xlim1[2] = MaxXValue
MaxYValue = max(y3)
MinYValue = min(y3)
ylim1 = c(1:2); ylim1[1] = MinYValue; ylim1[2] = MaxYValue
MaxZValue = max(N2CDFV1)
MinZValue = min(N2CDFV1)
zlim1 = c(1:2); zlim1[1] = MinZValue; zlim1[2] = MaxZValue
xTitle <- "X"
yTitle <- "Y"
zTitle <- "Z"
persp(x = x3, y = y3, z = N2CDFV1, 
  main = mTitle1, phi = phi1, theta = theta1, 
  xlim = xlim1, ylim = ylim1, zlim = zlim1,
  xlab = xTitle, ylab = yTitle, zlab = zTitle,
  axes = TRUE, scale=TRUE, box=TRUE)
persp(x = x3, y = y3, z = N2CDFV2, 
       main ="Bivariate CDF (pmvnorm)", col=NULL)
persp(x = x3, y = y3, z = N2CDFV3, 
       main ="Standard Bivariate CDF (pbivnorm)")
persp(x = x3, y = y3, z = Error12, 
       main ="Error (integral2 - pmvnorm)")
persp(x = x3, y = y3, z = Error13, 
       main ="Error (integral2 - pbivnorm)")
persp(x = x3, y = y3, z = Error23, 
       main ="Error (pmvnorm - pbivnorm)")
MaxError12 <- max(abs(Error12))
MaxError13 <- max(abs(Error13))
MaxError23 <- max(abs(Error23))
MaxError12; MaxError13; MaxError23
# PDF graph
MaxZValue = max(N2PDFV1)
MinZValue = min(N2PDFV1)
zlim1[1] = 0; zlim1[2] = MaxZValue
phi1 <- 25 # View height (higher number implies higher perspective)
theta1 <- 55 # Rotate left to right (higher number more right)
persp(x = x3, y = y3, z = N2PDFV1, 
      phi = phi1, theta = theta1, ticktype = "detailed", # main = mTitle1, 
      xlim = xlim1, ylim = ylim1, zlim = zlim1,
      xlab = 'x1', ylab = 'x2', zlab = 'f(x1,x2,p)',
      axes = TRUE, scale=TRUE, box=TRUE)
#
# Selected other 3D graphs found on the internet
#
require(grDevices) # for trans3d
## More examples in  demo(persp) !!
##                   -----------

# (1) The Obligatory Mathematical surface.
#     Rotated sinc function.

x <- seq(-10, 10, length.out = 30)
y <- x
f <- function(x, y) { r <- sqrt(x^2+y^2); 10 * sin(r)/r }
z <- outer(x, y, f)
z[is.na(z)] <- 1
op <- par(bg = "white")
persp(x, y, z, theta = 30, phi = 30, expand = 0.5, col = "lightblue")
persp(x, y, z, theta = 30, phi = 30, expand = 0.5, col = "lightblue",
      ltheta = 120, shade = 0.75, ticktype = "detailed",
      xlab = "X", ylab = "Y", zlab = "Sinc( r )"
) -> res
round(res, 3)

# (2) Add to existing persp plot - using trans3d() :

xE <- c(-10,10); xy <- expand.grid(xE, xE)
points(trans3d(xy[,1], xy[,2], 6, pmat = res), col = 2, pch = 16)
lines (trans3d(x, y = 10, z = 6 + sin(x), pmat = res), col = 3)

phi <- seq(0, 2*pi, length.out = 201)
r1 <- 7.725 # radius of 2nd maximum
xr <- r1 * cos(phi)
yr <- r1 * sin(phi)
lines(trans3d(xr,yr, f(xr,yr), res), col = "pink", lwd = 2)
## (no hidden lines)

# (3) Visualizing a simple DEM model

z <- 2 * volcano        # Exaggerate the relief (Auckland' Maunga Whau Volcano topography)
x <- 10 * (1:nrow(z))   # 10 meter spacing (S to N)
y <- 10 * (1:ncol(z))   # 10 meter spacing (E to W)
## Don't draw the grid lines :  border = NA
par(bg = "slategray")
persp(x, y, z, theta = 135, phi = 30, col = "green3", scale = FALSE,
      ltheta = -120, shade = 0.75, border = NA, box = FALSE)

# (4) Surface colours corresponding to z-values

par(bg = "white")
x <- seq(-1.95, 1.95, length.out = 30)
y <- seq(-1.95, 1.95, length.out = 35)
z <- outer(x, y, function(a, b) a*b^2)
nrz <- nrow(z)
ncz <- ncol(z)
# Create a function interpolating colors in the range of specified colors
jet.colors <- colorRampPalette( c("blue", "green") )
# Generate the desired number of colors from this palette
nbcol <- 100
color <- jet.colors(nbcol)
# Compute the z-value at the facet centres
zfacet <- z[-1, -1] + z[-1, -ncz] + z[-nrz, -1] + z[-nrz, -ncz]
# Recode facet z-values into color indices
facetcol <- cut(zfacet, nbcol)
persp(x, y, z, col = color[facetcol], phi = 30, theta = -30)
par(op)

